@@ -289,17 +289,19 @@ cc_network_panel_notify_enable_active_cb (GtkSwitch *sw,
 }
 
 static void
-on_property_change (GDBusProxy *proxy,
-                    GVariant   *changed_properties,
-                    GVariant   *invalidated_properties,
-                    gpointer    user_data)
+sync_airplane_mode_switch (CcNetworkPanel *panel)
 {
-        CcNetworkPanel *panel = CC_NETWORK_PANEL (user_data);
         GVariant *result;
         gboolean enabled;
 
-        result = g_dbus_proxy_get_cached_property (panel->priv->rfkill_proxy, "AirplaneMode");
+        result = g_dbus_proxy_get_cached_property (panel->priv->rfkill_proxy, "HasAirplaneMode");
+        enabled = g_variant_get_boolean (result);
 
+        gtk_widget_set_visible (GTK_WIDGET (panel->priv->rfkill_switch), enabled);
+        if (!enabled)
+                return;
+
+        result = g_dbus_proxy_get_cached_property (panel->priv->rfkill_proxy, "AirplaneMode");
         enabled = g_variant_get_boolean (result);
 	if (enabled != gtk_switch_get_active (panel->priv->rfkill_switch)) {
 		g_signal_handlers_block_by_func (panel->priv->rfkill_switch,
@@ -310,7 +312,15 @@ on_property_change (GDBusProxy *proxy,
 						 cc_network_panel_notify_enable_active_cb,
 						 panel);
 	}
+}
 
+static void
+on_property_change (GDBusProxy *proxy,
+                    GVariant   *changed_properties,
+                    GVariant   *invalidated_properties,
+                    gpointer    user_data)
+{
+        sync_airplane_mode_switch (CC_NETWORK_PANEL (user_data));
 }
 
 static void
@@ -328,6 +338,7 @@ got_rfkill_proxy_cb (GObject *source_object, GAsyncResult *res, gpointer user_da
 
         g_signal_connect (panel->priv->rfkill_proxy, "g-properties-changed",
                           G_CALLBACK (on_property_change), panel);
+        sync_airplane_mode_switch (panel);
 }
 
 static void
@@ -348,8 +359,9 @@ cc_network_panel_constructed (GObject *object)
         gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
         gtk_widget_set_visible (label, TRUE);
         widget = gtk_switch_new ();
+        gtk_widget_set_valign (widget, GTK_ALIGN_CENTER);
         gtk_label_set_mnemonic_widget (GTK_LABEL (label), widget);
-        gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 0);
+        gtk_box_pack_start (GTK_BOX (box), widget, FALSE, FALSE, 4);
         gtk_widget_show_all (box);
         panel->priv->rfkill_switch = GTK_SWITCH (widget);
         cc_shell_embed_widget_in_header (cc_panel_get_shell (CC_PANEL (panel)), box);
